const {app, Menu, Tray, BrowserWindow, ipcMain, dialog, shell, clipboard,crashReporter } = require('electron')
const http = require('http');
const path = require('path')
const fs = require("fs")
const {copyFile} = require("node:fs/promises")
const {fork, execFile,execSync} = require('child_process');
const Store = require('electron-store');
const log = require('electron-log');
const iconv = require('iconv-lite');
const {
    prepareFolder, getDbDir, getTmpDir, getUserDataDir, isPathExist,
    getDefaultRootFolder, ensureDir, autoSetPort, waitPortConnect,
    readFile, getIpList, getLogDir, upgrade, getSystemInfo, wait,
    tryDeleteFile,isWindowsRootDirectory, _find, checkExistBackupFile, joinPath, checkServerStatus
} = require("./tools");
const {storageKey, errorCode} = require("./constants");
const {t} = require("./i18n");
const crypto = require("crypto");
const store = new Store({
    name: "e-store"
});
let tray = null
const appStatus = {
    step:0,
    dbStatus: "",
    redisStatus: "",
    serverStatus: "",
    cacheDir: "",
    fileDir: "",
    exePath: "",
    serverPort: 8063,
    pgPort: 28032,
    redisPort: 28079,
    closeToTray: true,
}

let cmdEncoding="cp936"// 默认使用GBK编码

const appRef = {
    server: null,
    redisServer: null,
    mainWindow: null,
    tfTaskWindow: null,
    envWindow: null,
    resetSuperAdminPwdInfo: {}
}
const gotTheLock = app.requestSingleInstanceLock()
if (!gotTheLock) {
    app.quit()
}
const createWindow = ({startInTray}) => {
    let extra={}
    if(startInTray){
        extra.skipTaskbar=true
        extra.show =false
    }
    const win = new BrowserWindow({
        title: "MT Photos Server",
        maximizable: true,
        fullScreenable: false,
        resizable: true,
        width: 660,
        height: 580,
        titleBarOverlay: {
            color: '#2f3241',
            symbolColor: '#74b1be',
            height: 60
        },
        icon: path.join(__dirname, 'imgs/icon.png'),
        webPreferences: {
            preload: path.join(__dirname, 'preload.js'),
        },
        ...extra
    })

    win.loadFile('index.html')

    // win.webContents.openDevTools()
    Menu.setApplicationMenu(null)
    return win
}

app.on('second-instance', (event, commandLine, workingDirectory, additionalData) => {
    focusMainWindow()
})
app.whenReady().then(() => {
    setEncoding()
    regCrashReport()
    const locale = app.getLocale()
    const i18nLang = locale && locale.match('zh') !== null ? "zh" : "en";
    const startAtOpen = store.get(storageKey.startAtOpen) === "on" // requested-execution-level blocking startup
    const closeToTray = store.get(storageKey.closeToTray) !== "off"
    const startInTray = store.get(storageKey.startInTray) === "on" //开启时，默认最小化
    appStatus.closeToTray=closeToTray

    tray = new Tray(path.join(__dirname, 'imgs/icon.png'))
    const contextMenu = Menu.buildFromTemplate([
        {label: t('open', i18nLang), click: focusMainWindow},
        {label: t('openAtLogin', i18nLang), type: "checkbox", checked: startAtOpen, click: setOpenAtLogin},
        {label: t('closeToTray', i18nLang), type: "checkbox", checked: closeToTray, click: setCloseToTray},
        {label: t('startInTray', i18nLang), type: "checkbox", checked: startInTray, click: setStartInTray},
        {label: t('quit', i18nLang), click: appQuit},
    ])
    tray.setToolTip('MT Photos Server')
    tray.setContextMenu(contextMenu)
    tray.on('click', () => {
        focusMainWindow()
    })

    //网页事件，由preload.js中转
    ipcMain.handle('dialog:openFile', handleFileOpen)
    ipcMain.handle('dialog:handleOpenPath', handleOpenPath)
    ipcMain.handle('triggerApi', handleTriggerApi)
    ipcMain.handle('getServerStatus', handleGetServerStatus)
    ipcMain.handle('saveConfig', handleSaveConfig)

    const mainWindow = createWindow({startInTray})
    app.on('activate', () => {
        if (BrowserWindow.getAllWindows().length === 0) createWindow({startInTray:false})
    })
    mainWindow.on('closed', (event) => {
        appRef.mainWindow = null;
    });
    mainWindow.on('close', function (e) {
        e.preventDefault();
        mainWindow.hide();
        mainWindow.setSkipTaskbar(true);
        const notRunningStatus=[
            "CACHE_DIR_PATH",
            "CACHE_DIR_ACCESS",
            "FILE_DIR_PATH",
            "FILE_DIR_ACCESS",
            "hasNonASCII",
            "fail"
        ]
        if (appStatus.closeToTray === false || notRunningStatus.includes(appStatus.serverStatus)) {
            appQuit().catch((e)=>{
                log.error(e.toString())
            })
        }
    })
    appRef.mainWindow = mainWindow
    appStatus.step=1
    tryStartServer(mainWindow).catch((e)=>{
        log.error("tryStartServer",e)
    })

    checkServerStatus({
        appStatus,
        log,
        restartServer:()=>{
            log.log("_restartServer")
            _restartServer().catch(()=>{})
        }
    })

})
// app.on('window-all-closed', () => {
//
// })

function focusMainWindow() {
    if (!appRef.mainWindow) {
        return
    }
    if (appRef.mainWindow.isMinimized()) {
        appRef.mainWindow.restore()
        return;
    }
    appRef.mainWindow.show()
    appRef.mainWindow.setSkipTaskbar(false)
    appRef.mainWindow.focus()
}

async function appQuit() {
    try {
        await stopPg()
    } catch (e) {

    }
    if (appRef.tfTaskWindow) {
        appRef.tfTaskWindow.destroy()
    }
    if (appRef.envWindow) {
        appRef.envWindow.destroy()
    }
    if (appRef.mainWindow) {
        appRef.mainWindow.destroy()
    }
    tray.destroy()
    app.quit()
}
function regCrashReport(){
    const topDir = app.getPath("userData")
    app.setPath('crashDumps', path.join(topDir,"crashLogs"))
    crashReporter.start({
        uploadToServer: false
    });
}
function setOpenAtLogin(menu) {
    const {checked} = menu
    store.set(storageKey.startAtOpen, checked ? "on" : "off")
    app.setLoginItemSettings({
        openAtLogin: checked
    })
}
function setCloseToTray(menu) {
    const {checked} = menu
    store.set(storageKey.closeToTray, checked ? "on" : "off")
    appStatus.closeToTray=checked
}
function setStartInTray(menu) {
    const {checked} = menu
    store.set(storageKey.startInTray, checked ? "on" : "off")
    appStatus.startInTray=checked
}
function tryStartServer(mainWindow) {
    return new Promise((resolve, reject) => {
        initCheck().then(({code, pgPort, redisPort,redisDir}) => {
            appStatus.step=2
            if (code) {
                appStatus.serverStatus = code
                //未完成初始化设置
                resolve()
            } else {
                log.log("pgPort", pgPort, "redisPort", redisPort)
                startAllService({pgPort, redisPort,redisDir}).then(resolve).catch(reject)
            }
        }).catch((e) => {
            appStatus.step=-2
            appStatus.serverStatus="fail"
            reject(e)
        })
    })

}

async function initCheck() {
    const exePwd = app.getPath("exe")
    const hasNonASCII = /[^\x00-\x7f]/g.test(exePwd)
    if (hasNonASCII) {
        appStatus.serverStatus = "hasNonASCII"
        appStatus.exePath = exePwd
        return {code:"hasNonASCII"}
    }
    appStatus.serverStatus = ""
    appStatus.step=21
    const {dbDir,redisDir} = await prepareFolder()

    const fileDir = store.get(storageKey.fileDir)
    if (!fileDir) {
        return {code: errorCode.fileDirPath}
    }
    appStatus.fileDir = fileDir
    const cacheDir = store.get(storageKey.cacheDir)
    if (!cacheDir) {
        return {code: errorCode.cacheDirPath}
    }
    if (await isPathExist(cacheDir) === false) {
        return {code: errorCode.cacheDirAccess}
    }
    appStatus.cacheDir = cacheDir
    const pgConfPath = path.join(dbDir, 'postgresql.conf');
    appStatus.step=22
    const pgExists = await isPathExist(pgConfPath);
    if (!pgExists) {
        log.info('===== initDb =====');
        appStatus.dbStatus = "initDb"
        appStatus.step=221
        await initPg()
        appStatus.step=222
    }
    const redisConfPath = path.join(redisDir, 'redis.windows.conf');
    appStatus.step=23
    const redisExists = await isPathExist(redisConfPath);
    if (!redisExists) {
        log.info('===== initRedis =====');
        appStatus.dbStatus = "initDb"
        appStatus.step=231
        await initRedis(redisDir)
        appStatus.step=232
    }
    appStatus.step=24
    const {pgPort, redisPort} = await autoSetPort()
    appStatus.step=25
    return {pgPort, redisPort,redisDir}
}

async function startAllService({pgPort, redisPort,redisDir}) {
    appStatus.dbStatus = ""
    appStatus.redisStatus = ""
    appStatus.serverStatus = ""
    appStatus.step=3
    await startRedis(redisPort,redisDir)
    appStatus.step=4
    await startPg(pgPort)
    appStatus.step=5
    await startServer({pgPort, redisPort,redisDir})
    appStatus.step=6

    // setTimeout(()=>{
    //     let pwd="C:\\mt-photos-files\\xxx.exe"
    //     startApiServer("facial_api",pwd,{
    //         onLog:()=>{
    //
    //         }
    //     })
    // },5000)
}

async function startServer({pgPort, redisPort,redisDir}) {
    const dataDir = getUserDataDir()
    const tmpDir = getTmpDir()
    const {fileDir, cacheDir} = appStatus
    const envData = getEnvData()
    const server = fork('launcher.js', {
        cwd: path.join(__dirname, 'server'),
        silent: true,
        env: {
            SERVER_TYPE: "electron",
            APP_ENV: "prod",
            NODE_ENV: "production",
            JWT_SECRET_PATH: dataDir,
            root_path: dataDir,
            CACHE_DIR_PATH: cacheDir,
            TMP_DIR_PATH: tmpDir,
            UPLOAD_DIR_PATH: fileDir,
            TF_TASK_DIR_PATH:path.join(__dirname, 'tfTask'),
            REDIS_HOST: "127.0.0.1",
            REDIS_PORT: "" + redisPort,
            POSTGRES_HOST: "127.0.0.1",
            POSTGRES_PORT: "" + pgPort,
            POSTGRES_DATABASE: "postgres",
            POSTGRES_USER: "postgres",
            POSTGRES_PASSWORD: "4CB6B76525B0841ED0C6F0344A4B846ABFECF5CDBB01D122E7A5C37E47D13D57",
            ...envData
        }
    });
    server.stdout.on('data', (data) => {
        log.info(data.toString());
    });
    server.stderr.on('data', (data) => {
        log.error(data.toString());
    });
    server.on("exit", (code) => {
        if (code === 0) {
            appRef.server = null
            restartServer({pgPort, redisPort}).catch((e) => {
                log.error(e)
            })
        }else if (code === 1) {
            appRef.server = null
            clearRedisFile(redisDir).finally(() => {
                _restartServer()
            })
        }
    })
    server.on("message", (data) => {
        if (data && data.type) {
            switch (data.type) {
                case "pgDump":
                    pgDump(data).then((distPath) => {
                        server.send({resultFor: "pgDump", distPath})
                    }).catch((e) => {
                        server.send({resultFor: "pgDump", msg: e.toString()})
                    })
                    break
                case "pgAutoBackup":
                    pgAutoBackup(data).then((distPath) => {
                        server.send({resultFor: "pgAutoBackup", distPath})
                    }).catch((e) => {
                        server.send({resultFor: "pgAutoBackup", msg: e.toString()})
                    })
                    break
                case "serverStart":
                    appStatus.serverPort = data.port
                    appStatus.serverStatus = "running"
                    break
                case 'upgrade':
                    const dist = path.join(__dirname, "..")
                    upgrade(data, dist).then((res) => {
                        const {msg} = res
                        server.send({resultFor: data.type, msg})
                    }).catch((e) => {
                        log.log("upgrade catch e", e.toString())
                        server.send({resultFor: data.type, msg: e.toString()})
                    })
                    break
                case "resetSuperAdminPwd":
                    appRef.resetSuperAdminPwdInfo = data
                    break
                case "startTfTask":
                    const {taskType, authCode, socketPort} = data
                    openBroTask({taskType, authCode, socketPort}).then((win) => {

                    }).catch((e) => {
                        console.log("catch error:", e)
                    })
                    break
                default:
                    log.log("message:", data)
            }
        }
    })
    appRef.server = server
    return server
}

function stopServer() {
    return new Promise((resolve, reject) => {
        if (appRef.server) {
            appRef.server.on('exit', (code) => {
                resolve()
            })
            appRef.server.kill("SIGTERM")
        } else {
            resolve()
        }
    })
}

async function restartServer({pgPort, redisPort}) {
    // await restartRedis(redisPort,redisDir)
    await stopServer()
    await startServer({pgPort, redisPort})
}

async function initRedis(redisDir){
    const sourceDir=path.join(__dirname, 'extra', 'redis')
    await copyFile(path.join(sourceDir,"redis-server.exe"),path.join(redisDir,"redis-server.exe"))
    await copyFile(path.join(sourceDir,"redis.windows.conf"),path.join(redisDir,"redis.windows.conf"))
}
async function clearRedisFile(redisDir){
    await stopRedis()
    if(!redisDir){
        log.error("redisDir empty",redisDir)
        return
    }
    await tryDeleteFile(path.join(redisDir,"dump.rdb"))
    await tryDeleteFile(path.join(redisDir,"redis-server.log"))
}
async function startRedis(redisPort,redisDir) {
    execFileP({
        file: 'redis-server.exe',
        options: ['redis.windows.conf', '--port', redisPort],
        cwd: redisDir,
        refName: "redisServer"
    }).catch((e) => {
        if(appStatus.redisStatus === ""){
            return //stopRedis()
        }
        log.error("startRedis e", e)
    })
    await waitPortConnect(redisPort)
    appStatus.redisStatus = "running"
}

function stopRedis() {
    return new Promise((resolve, reject) => {
        appStatus.redisStatus = ""
        if (appRef.redisServer && appRef.redisServer.stdin) {
            appRef.redisServer.on('exit', (code) => {
                resolve()
            })
            appRef.redisServer.stdin.end();
            appRef.redisServer.kill("SIGTERM");
        } else {
            resolve()
        }

    })
}

async function restartRedis(port,redisDir) {
    await stopRedis()
    await startRedis(port,redisDir)
}

async function initPg() {
    const {dbDir} = await prepareFolder()
    await tryDeleteFile(getPgAutoConf(dbDir))
    return await execFileP({
        file: "pgsql/bin/initdb.exe",
        options: ['--pgdata=' + dbDir, '--username=postgres', '--pwfile=db_pwd', '--encoding=UTF8', '--locale=C', '--auth=md5'],
        cwd: path.join(__dirname, 'extra', 'postgresql'),
    })
}
function getPgAutoConf(dbDir){
    return path.join(dbDir, 'postgresql.auto.conf')
}
async function startPg(pgPort) {
    const dbDir = getDbDir()
    try {
        await stopPg()
    } catch (e) {
        log.error("stopPg error", e)
        await wait(1000)
    }
    await changePgConf(dbDir, pgPort)
    const logFile = path.join(getLogDir(), "pgsql.log")
    appStatus.pgPort = pgPort
    execFileP({
        file: "pgsql\\bin\\pg_ctl.exe",
        options: ['--pgdata=' + dbDir, `--log=${logFile}`, 'start'],
        cwd: path.join(__dirname, 'extra', 'postgresql'),
    }).catch((e) => {
        log.error("startPg e", e)
    })
    await waitPortConnect(pgPort)
    appStatus.dbStatus = "running"
}

async function stopPg() {
    //停止数据库
    const dbDir = getDbDir()
    const pidPath = path.join(dbDir, 'postmaster.pid');
    const pidExists = await isPathExist(pidPath);
    if (!pidExists) {
        //检查pid是否存在，不存在就不用停止
        return
    }
    return await execFileP({
        file: "pgsql\\bin\\pg_ctl.exe",
        options: ['--pgdata=' + dbDir, 'stop'],
        cwd: path.join(__dirname, 'extra', 'postgresql'),
    })
}

async function pgDump(msg) {
    const saveDir = store.get(storageKey.fileDir) //备份文件夹
    const now = new Date()
    let fileName = `mt_photos_db_dump_${dateToText(now)}.sql`
    if(msg.data && msg.data.version){
        fileName = `mt_photos_v${msg.data.version}_db_dump_${dateToText(now)}.sql`
    }
    const cwd = path.join(__dirname, 'extra', 'postgresql')
    const db_pwd = await readFile(path.join(cwd, "db_pwd"))
    const distPath = path.join(saveDir, fileName)
    await execFileP({
        file: "pgsql\\bin\\pg_dump.exe",
        options: ['-c', '-h', '127.0.0.1', '-U', 'postgres', '-d', 'postgres', '-p', appStatus.pgPort, '-f', distPath],
        cwd,
        env: {
            PGPASSWORD: db_pwd
        }
    })
    return distPath
}

async function pgAutoBackup(msg) {
    if(!msg || !msg.data  || !msg.data.fileName){
        return
    }
    const {subFolderName,fileName,file_num,interval}=msg.data
    const nowDate = new Date()
    const saveDir = store.get(storageKey.fileDir)
    const cwd = joinPath(__dirname, 'extra', 'postgresql')
    const db_pwd = await readFile(joinPath(cwd, "db_pwd"))
    const distFolderPath=joinPath(saveDir,subFolderName)
    const distPath = joinPath(distFolderPath, fileName)
    await ensureDir(distFolderPath)

    const existsFiles = await _find(distFolderPath)
    const {needAdd, needDeleteFiles} = checkExistBackupFile({interval, file_num, nowDate, existsFiles})

    if(needAdd) {
        await execFileP({
            file: "pgsql\\bin\\pg_dump.exe",
            options: ['-c', '-h', '127.0.0.1', '-U', 'postgres', '-d', 'postgres', '-p', appStatus.pgPort, '-f', distPath],
            cwd,
            env: {
                PGPASSWORD: db_pwd
            }
        })
        log.log("pg_dump success", joinPath(distFolderPath,fileName) )
    }
    for (let i = 0; i < needDeleteFiles.length; i++) {
        const deleteFilePath = needDeleteFiles[i];
        await tryDeleteFile(deleteFilePath);
    }
    return {needAdd,distPath}
}

async function pgRestore() {
    const fileDir = store.get(storageKey.fileDir) //备份文件夹
    const {canceled, filePaths} = await dialog.showOpenDialog({
        defaultPath: fileDir,
        properties: ["openFile"]
    })
    if (canceled || filePaths.length !== 1) {
        return "canceled"
    }
    const filePath = filePaths[0]
    const cwd = path.join(__dirname, 'extra', 'postgresql')
    const db_pwd = await readFile(path.join(cwd, "db_pwd"))
    await execFileP({
        file: "pgsql\\bin\\psql.exe",
        options: ['-h', '127.0.0.1', '-U', 'postgres', '-p', appStatus.pgPort, '-f', filePath],
        cwd,
        env: {
            PGPASSWORD: db_pwd
        }
    })
    await restartServer({pgPort: appStatus.pgPort, redisPort: appStatus.redisPort})
    return filePath
}

async function resetAdminPwd() {
    const tmpDir = getTmpDir();
    await ensureDir(tmpDir);
    const check = crypto.randomUUID();
    fs.writeFileSync(path.join(tmpDir, 'reset_' + check.slice(0, 8)), check);
    const {n} = await patch(`http://127.0.0.1:${appStatus.serverPort}/users/resetSuperAdminPwd`, {check});
    if (n === 1) {
        if (appRef.resetSuperAdminPwdInfo && appRef.resetSuperAdminPwdInfo.password) {
            clipboard.writeText(appRef.resetSuperAdminPwdInfo.password)
        }
        return appRef.resetSuperAdminPwdInfo
    } else {
        return {msg: 'fail'}
    }
}

function patch(url, data) {
    return new Promise((resolve, reject) => {
        const options = {
            method: 'PATCH',
            headers: {'Content-Type': 'application/json'}
        };
        const req = http.request(url, options, (res) => {
            let data = '';
            res.on('data', (chunk) => {
                data += chunk;
            });
            res.on('end', () => {
                resolve(JSON.parse(data));
            });
        });

        req.on('error', (err) => {
            reject(err);
        });
        req.write(JSON.stringify(data));
        req.end();
    })
}

function dateToText(time) {
    let d = {
        year: time.getFullYear(),
        month: time.getMonth() + 1,
        day: time.getDate(),
        h: time.getHours(),
        m: time.getMinutes(),
        s: time.getSeconds()
    }
    for (let key in d) {
        if (d[key] < 10) {
            d[key] = '0' + d[key]
        }
    }
    return `${d.year}-${d.month}-${d.day}_${d.h}_${d.m}_${d.s}`
}

function setEncoding(){
    try {
        const output = execSync('chcp', { encoding: 'utf8' });
        if(output.includes('936')===false){
            cmdEncoding="utf8"
        }
    } catch (err) {
        console.error('Failed to get code page:', err);
    }
}

function iconvDecode(str = '') {
    return iconv.decode(Buffer.from(str, "binary"), cmdEncoding);
}

function execFileP({file, options = [], cwd, env = {}, refName}) {
    return new Promise((resolve, reject) => {
        const process = execFile(file, options, {
            cwd,
            env,
            encoding: 'binary'
        }, (error, stdout, stderr) => {
            if (error && error.message) {
                reject(iconvDecode(error.message))
                return;
            }
            if (stderr) {
                reject(iconvDecode(stderr))
                return
            }
            resolve(iconvDecode(stdout))
        });

        if (refName) {
            appRef[refName] = process
        }
    })
}


async function handleFileOpen(channel, defaultPath) {
    let fileDir = getDefaultRootFolder()

    if (defaultPath && defaultPath.length) {
        fileDir = defaultPath
    }
    const {canceled, filePaths} = await dialog.showOpenDialog({
        defaultPath: fileDir,
        properties: ["openDirectory"]
    })
    if (!canceled) {
        return filePaths[0]
    }
}

function handleOpenPath(channel, type) {
    let folder = ""
    if (type === "fileDir") {
        folder = store.get(storageKey.fileDir)
    } else if (type === "cacheDir") {
        folder = store.get(storageKey.cacheDir)
    } else if (type === "log") {
        folder = getLogDir()
    }
    if (folder) {
        shell.openPath(folder)
    }
}

async function handleGetServerStatus(channel,type) {
    if (type !== "all") {
        return {...appStatus}
    }
    let {fileDir, cacheDir} = appStatus
    let rootDir = ""
    if (fileDir === "" || cacheDir === "") {
        rootDir = await getDefaultRootFolder()
    }
    if (fileDir === "") {
        fileDir = path.join(rootDir, "upload")
    }
    if (cacheDir === "") {
        cacheDir = path.join(rootDir, "cache")
    }
    return {
        ...appStatus,
        fileDir,
        cacheDir,
        ipList: getIpList()
    }
}

async function handleSaveConfig(channel, data) {
    if (data) {
        const {fileDir, cacheDir} = data
        if(isWindowsRootDirectory(fileDir) || isWindowsRootDirectory(cacheDir)){
            return {msg:"ROOT_DIR_ERROR"}
        }
        if (fileDir) {
            store.set(storageKey.fileDir, fileDir)
            appStatus.fileDir = fileDir
            await ensureDir(fileDir)
        }
        if (cacheDir) {
            store.set(storageKey.cacheDir, cacheDir)
            appStatus.cacheDir = cacheDir
            await ensureDir(cacheDir)
        }
    }
    tryStartServer(appRef.mainWindow).catch((e) => {
        log.error(e)
    })
}

async function _restartServer(){
    try {
        await stopPg()
    } catch (e) {

    }
    app.relaunch({})
    app.exit(0)
}

async function handleTriggerApi(channel, type, data) {
    switch (type) {
        case "openExternal":
            if (data && data.url) {
                shell.openExternal(data.url)
            }
            break;
        case "restartServer":
            await _restartServer()
            break;
        case "pgRestore":
            return await pgRestore()
            break;
        case "resetAdminPwd":
            return await resetAdminPwd()
            break;
        case "openEnvConfig":
            const win = await openEnvConfig()
            break;
        case "getEnvConfig":
            return getEnvData()
            break;
        case "saveEnvConfig":
            return saveEnvConfig(data)
            break;
        case "systemInfo":
            let info = await getSystemInfo()
            return JSON.stringify(info, null, 2)
            break;
        case "debugInfo":
            await openDebugInfo({})
            return 1
            break;
        case "quit":
            await appQuit()
            return 1
            break;
        default:
            log.warn("handleTriggerApi type", type)
    }
}

function changePgConf(dbDir, port) {
    return new Promise((resolve, reject) => {
        let work_mem = 50
        const content = `# overwritten by main.js
port= ${port}
work_mem = '${work_mem}MB'`
        fs.writeFile(getPgAutoConf(dbDir), content, 'utf8', (err) => {
            if (err) {
                reject(err);
                return;
            }
            resolve();
        });
    });
}

function openBroTask({taskType, authCode, socketPort}) {
    return new Promise((resolve, reject) => {
        if (appRef.tfTaskWindow) {
            appRef.tfTaskWindow.close()
            appRef.tfTaskWindow = null
        }

        const win = new BrowserWindow({
            width: 800,
            height: 600,
            show: false,
            webPreferences: {
                nodeIntegration: true,
                preload: path.join(__dirname, 'tfTaskPreload.js')
            }
        })
        win.loadFile(path.join(__dirname, 'tfTask', taskType === "fileClassify" ? 'classify.html' : 'faceReg.html'), {
            query: {
                serverPort: appStatus.serverPort,
                authCode,
                socketPort
            }
        })

        win.on("closed", () => {
            appRef.tfTaskWindow = null
        })
        const menu = Menu.buildFromTemplate([
            {
                label: app.name,
                submenu: [
                    {
                        click: () => {
                            win.webContents.reload()
                        },
                        label: 'reload'
                    },
                    {
                        click: () => {
                            win.webContents.openDevTools()
                        },
                        label: 'devtool'
                    },
                ]
            }

        ])

        win.setMenu(menu)

        appRef.tfTaskWindow = win
        resolve(win)
    })
}

function openDebugInfo({type}) {
    return new Promise((resolve, reject) => {
        const win = new BrowserWindow({
            width: 800,
            height: 600,
            show: true,
            webPreferences: {
            }
        })
        win.loadFile(path.join(__dirname, 'tfTask','debugInfo.html'), {
            query: {

            }
        })
        const menu = Menu.buildFromTemplate([
            {
                label: "Menu",
                submenu: [
                    {
                        click: () => {
                            win.webContents.reload()
                        },
                        label: 'reload'
                    },
                    {
                        click: () => {
                            win.webContents.openDevTools()
                        },
                        label: 'devtool'
                    },
                ]
            }

        ])

        win.setMenu(menu)
        resolve(win)
    })
}

function openEnvConfig() {
    return new Promise((resolve, reject) => {
        if (appRef.envWindow) {
            appRef.envWindow.close()
            appRef.envWindow = null
        }
        const win = new BrowserWindow({
            width: 650,
            height: 420,
            show: true,
            modal: true,
            parent: appRef.mainWindow,
            webPreferences: {
                // nodeIntegration:true,
                preload: path.join(__dirname, 'preload.js')
            }
        })
        win.loadFile(path.join(__dirname, 'env.html'), {query: {}})
        // win.webContents.openDevTools()

        win.on("closed", () => {
            appRef.envWindow = null
        })
        appRef.envWindow = win
        resolve(win)
    })
}

async function saveEnvConfig(newConfig) {
    if (!newConfig) {
        newConfig = {}
    }
    store.set(storageKey.env, newConfig)
    if (appRef.envWindow) {
        appRef.envWindow.close()
        appRef.envWindow = null
    }
    await restartServer({pgPort: appStatus.pgPort, redisPort: appStatus.redisPort})
}

function getEnvData() {
    const envData = store.get(storageKey.env)
    return envData ? envData : {}
}

function startApiServer(refName, exePath,{onLog}) {
    const _cwd = path.dirname(exePath)
    const _fileName = path.basename(exePath)
    return new Promise((resolve, reject) => {
        const process = execFile(_fileName, [], {
            cwd:_cwd,
            env:[],
            encoding: 'binary'
        }, (error, stdout, stderr) => {
            if (error && error.message) {
                reject(iconvDecode(error.message))
                return;
            }
            if (stderr) {
                reject(iconvDecode(stderr))
                return
            }
            resolve(iconvDecode(stdout))
        });

        if (refName) {
            appRef[refName] = process
        }
        process.unref();
        process.stdout.on('data', function(data) {
            onLog && onLog(data.toString());
        });

    })
}
